
-- Temporary tables vs table variables

create procedure table_variable_proc 
as 
begin
declare @table_variable table(col1 int, col2 varchar(128));
      declare @i int;
      set @i = 0;
      while (@i < 100000)
      begin
insert into @table_variable values(@i, convert(varchar(128), @i));
            set @i = @i + 1;
      end
select * from @table_variable tv join test_table on tv.col1 = test_table.col1;
end
go
 
exec table_variable_proc
go
 
-- Now let us rewrite the same example with temporary tables to compare and contrast the two approaches:
 
create procedure temp_table_proc 
as 
begin
create table #table_name(col1 int, col2 varchar(128));
      declare @i int;
      set @i = 0;
      while (@i < 100000)
      begin
insert into #table_name values(@i, convert(varchar(128), @i));
            set @i = @i + 1;
      end
select * from #table_name join test_table on #table_name.col1 = test_table.col1;
end
go
 
exec temp_table_proc
go
 
-- Now query the DMVs to get the query plan and average CPU time:
-- The temporary table outperforms the table variable query
 
select total_worker_time/execution_count as avg_cpu_time,
substring(st.text, (qs.statement_start_offset/2) + 1,
((case statement_end_offset 
when -1 
      then datalength(st.text) 
else 
      qs.statement_end_offset 
end 
- qs.statement_start_offset)/2) + 1) as statement_text
, cast(query_plan as xml)
from sys.dm_exec_query_stats qs
cross apply sys.dm_exec_sql_text(qs.sql_handle) st
cross apply sys.dm_exec_text_query_plan(qs.plan_handle, qs.statement_start_offset, qs.statement_end_offset) 
order by total_worker_time/execution_count desc;
go

-- See recompiles in temp tables
-- Deferred compile because table does not exist yet
create procedure DemoProc1 
as 
begin
   create table #t1(a int, b int); 
   insert into #t1 values(1,2);
   select * from #t1;
end
go
 
-- Enable the SP:Recompile
-- and SP:StmtRecompile events in profiler. 
exec DemoProc1
go 


-- Now consider a case when the temporary table is referenced 
-- in a second stored procedure as below:
 
create procedure DemoProc1 
as 
begin
   create table #t1(a int, b int); 
   insert into #t1 values(1,2);
   exec DemoProc2;      
end
go
 
create procedure DemoProc2
as 
begin
      select * from #t1;
end
go
 
-- Now enable the SP:Recompile and SP:StmtRecompile events in profiler. 
 
exec DemoProc1
go 
 
exec DemoProc1
go

-- Now let us make a slight variation to DemoProc1 as illustrated below:

create procedure DemoProc1 
as 
begin
   create table #t1(a int, b int); 
   insert into #t1 values(1,2);
   exec DemoProc2;
   exec DemoProc2;      
end
go
 
create procedure DemoProc2
as 
begin
      select * from #t1;
end
go
 
exec DemoProc1
go 
 
-- Notice that the second execution of DemoProc2 inside 
-- DemoProc1 causes no recompiles.

-- One of the most common reasons for recompiles of queries 
-- with temporary tables is row count modification. 
-- Consider the example below:
 
create procedure RowCountDemo 
as 
begin 
    create table #t1 (a int, b int);
    declare @i int;
    set @i = 0;
    while (@i < 20) 
    begin
            insert into #t1 values (@i, 2*@i - 50);
select a from #t1 where a < 10 or ((b > 20 or a >=100) and (a < 10000)) group by a ;
            set @i = @i + 1; 
    end 
end 
go 

exec RowCountDemo 
go

