
-- Trace Cache Hits, Misses, etc
-- No Miss event because its immediately followed by a CacheInsert
-- this is a decision made by the team (leave out miss, followed by insert)
select * from t1 where col1 = 5
go

exec sales_report @country_id = 2 
go

-- Now let us consider an example where we will fire the SP:Cache Miss Event: 
 
create table t1(col1 int primary key, col2 int, col3 int)
go

-- flush cache 
dbcc freeproccache
go

-- Consider a case we query a table in an auto-close database 
-- which results in plan cache flush:
alter database test_db set auto_close on
go
 
select * from test_db.dbo.test_table
go

/*
The SP:Cache Hit event is fired when we find the compiled plan and/or the MXC in the cache for query submitted for execution. If we find the MXC in the cache from previous execution, then we will fire SP:Cache Hit event with eventsubclass: 1- Execution Context Hit. For example:
*/
 
select * from t1 where col1 = 5;
go
 
select * from t1 where col1 = 5;
go

-- If the MXC has been deleted due to memory pressure, or it was never cached, or if its not locally available on the same node in a multi-node machine, then we just fire the SP:Cache Hit event with Eventsubclass 2- Compplan Hit. For example:
 
select * from t1 where col1 = 5;
go
 
select * from t1 where col1 = 6;
go

-- The SP:Recompile and SQL:StmtRecompile trace event. . 
-- The plan cache related trace events and their different eventsubclasses can be found using:
 
select distinct * from sys.trace_subclass_values 
where trace_event_id in (34, 35, 36, 37, 38) 
and trace_column_id = 21 
order by trace_event_id, subclass_value
go


 



