
-- section 12 - stats

-- The performance statistics trace event has 4 subclasses: 
select distinct * from sys.trace_subclass_values 
where trace_event_id = 165
and trace_column_id = 21 
order by trace_event_id, subclass_value
go

 
-- use it
create proc sales_report @country_id int
as begin
      select * from sales where country_id = @country_id
end
go
 
-- Now let us suppose this procedure is executed with parameter value 2 to 
-- generate the sales report for country id 2:
 
exec sales_report 2
go

-- Now let us check the DMVs to find get the query plan and the execution time 
-- statistics:
 
select total_worker_time/execution_count as avg_cpu_time,
substring(st.text, (qs.statement_start_offset/2) + 1,
((case statement_end_offset 
when -1 
      then datalength(st.text) 
else 
      qs.statement_end_offset 
end 
- qs.statement_start_offset)/2) + 1) as statement_text
, cast(query_plan as xml)
from sys.dm_exec_query_stats qs
cross apply sys.dm_exec_sql_text(qs.sql_handle) st
cross apply sys.dm_exec_text_query_plan(qs.plan_handle, qs.statement_start_offset, qs.statement_end_offset) 
order by total_worker_time/execution_count desc;
go

-- Analyze the performance stats
create procedure analyze_perf_stats_trace_data 
as
begin
 
      if object_id('trace_data', 'U') is not null
            drop table trace_data;
 
      if object_id('perf_stats_data_inserted', 'U') is not null
            drop table perf_stats_data_inserted;
 
      if object_id('perf_stats_data_deleted', 'U') is not null
            drop table perf_stats_data_deleted;
      
if object_id('perf_stats_inserted_with_corresponding_deleted_object_events', 'U') is not null
drop table perf_stats_inserted_with_corresponding_deleted_object_events;
 
if object_id('perf_stats_deleted_with_corresponding_inserted_object_events', 'U') is not null
drop table perf_stats_deleted_with_corresponding_inserted_object_events;
 
      if object_id('perf_stats_removed_plan', 'U') is not null
            drop table perf_stats_removed_plan;
 
create table perf_stats_inserted_with_corresponding_deleted_object_events (TextData xml, EventClass int, 
EventSubClass int, SqlHandle varbinary(64),
PlanHandle varbinary(64), StartTime datetime, 
EventSequence int, PlanGenerationNumber bigint, 
IntegerData2 int, Offset int);
 
create table perf_stats_deleted_with_corresponding_inserted_object_events (TextData xml, EventClass int, 
EventSubClass int, SqlHandle varbinary(64),
PlanHandle varbinary(64), StartTime datetime, 
EventSequence int, PlanGenerationNumber bigint, 
IntegerData2 int, Offset int);
 
      ---read all the trace data from file
      select * into trace_data from 
      ::fn_trace_gettable ('c:\temp\perfstats.trc', default);
 
      ---select all rows corresponding to all object events
      ---cast the Sql and Plan Handle into varbinary(64)
      select convert(xml, TextData) as TextData, EventClass, 
      EventSubClass,  convert(varbinary(64), SqlHandle) as SqlHandle, 
      convert(varbinary(64), PlanHandle) as PlanHandle, 
      StartTime, EventSequence, BigintData1, 
      IntegerData2, Offset
      into perf_stats_data_inserted
      from trace_data 
      where EventClass = 165 and 
      EventSubClass = 1;
 
      ---select all rows corresponding to query stats events 
      ---these events are generated when an plan is removed from cache
      ---for object and adhoc plans
      select convert(xml, TextData) as TextData, EventClass, 
      EventSubClass, convert(varbinary(64), SqlHandle) as SqlHandle, 
      convert(varbinary(64), PlanHandle) as PlanHandle, 
      StartTime, EventSequence, BigintData1,
      IntegerData2, Offset 
      into perf_stats_data_deleted
      from trace_data 
      where EventClass = 165 and 
      EventSubClass = 3;
 
/* for every query statistics (plan deleted event) find the corresponding plan inserted events (object plans only). The plan inserted event should have lower event sequence number tha the QE stats event so we find plan insert events that occured earlier than the QE stats events. It should also have the 
same plan handle as the QE stats event, statement start and end offset as the QE stats event. The QE stats and object plan events should also have the same plan generation number (bigintdata1). By matching plan generation numbers
we avoid duplicate rows from recompiles.
      */
declare @TextData xml, @EventClass int, @EventSubClass int, @SqlHandle varbinary(64), @PlanHandle varbinary(64), @StartTime datetime, @EventSequence int, @PlanGenerationNum bigint,
      @IntegerData2 int, @Offset int;
declare event_cursor cursor for select * from perf_stats_data_deleted;
      open event_cursor;
fetch next from event_cursor into @TextData, @EventClass, @EventSubClass, @SqlHandle, @PlanHandle, @StartTime, @EventSequence, @PlanGenerationNum, @IntegerData2, @Offset;
      while @@fetch_status =0
      begin
insert into perf_stats_inserted_with_corresponding_deleted_object_events 
select * from perf_stats_data_inserted where EventSequence < @EventSequence 
and PlanHandle = @PlanHandle and BigIntData1 = @PlanGenerationNum
            and IntegerData2 = @IntegerData2 and Offset = @Offset;
if exists(select * from perf_stats_data_inserted where EventSequence < @EventSequence 
            and PlanHandle = @PlanHandle) 
insert into perf_stats_deleted_with_corresponding_inserted_object_events values(@TextData, @EventClass, @EventSubClass, @SqlHandle,
@PlanHandle, @StartTime, @EventSequence, @PlanGenerationNum, @IntegerData2, @Offset)
fetch next from event_cursor into @TextData, @EventClass, @EventSubClass, @SqlHandle,
@PlanHandle, @StartTime, @EventSequence, @PlanGenerationNum, @IntegerData2, @Offset;
      end
      close event_cursor;
      deallocate event_cursor;
            
---compute the CPU time of the deleted plan from the QE stats 
---event Text Data, plan handle, sql handle, and query plan that ---was removed and save the data into a table
      
select 
d.TextData.value('(/QueryExecutionStats/WorkerTime/@Total)[1]', 'bigint') /
d.TextData.value('(/QueryExecutionStats/GeneralStats/@ExecutionCount)[1]', 'bigint')
      as avg_cpu_time_removed,
      i.TextData as query_plan_removed, 
      d.PlanGenerationNumber as plangenerationnum_removed,
      d.PlanHandle as planhandle_removed, 
      d.SqlHandle as sqlhandle
      into perf_stats_removed_plan
from 
perf_stats_deleted_with_corresponding_inserted_object_events d, perf_stats_inserted_with_corresponding_deleted_object_events i
      where i.PlanHandle = d.PlanHandle;
 
      ---note that plan generation numbers >2 indicate recompiles
      select * from perf_stats_removed_plan;
 
/* Check if sys.dm_exec_query_stats has any rows with same Sql Handle and plan generation one greater than the deleted plan
If yes, get the CPU time, sql text, query plan, plan handle, sql handle compare results from this query with rows from perf_stats_removed_plan to indentify queries that are running slower */
      declare @Sql_Handle varbinary(64), @PlanGenNum bigint;
declare event_cursor cursor for 
select distinct sqlhandle, plangenerationnum_removed from perf_stats_removed_plan;
      open event_cursor;
      fetch next from event_cursor into @Sql_Handle, @PlanGenNum;
      while @@fetch_status =0
      begin
if exists(select * from sys.dm_exec_query_stats where sql_handle = @Sql_Handle and plan_generation_num <= @PlanGenNum + 1)
select total_worker_time/execution_count as avg_cpu_time_current,
            substring(st.text, (qs.statement_start_offset/2) + 1,
            ((case statement_end_offset 
            when -1 
                  then datalength(st.text) 
            else 
                  qs.statement_end_offset 
            end 
            - qs.statement_start_offset)/2) + 1) as statement_text, 
            cast(query_plan as xml) as query_plan_current,
            plan_generation_num as plan_generation_num_current, 
            plan_handle as planhandle_current,
            sql_handle as sqlhandle
            from sys.dm_exec_query_stats qs 
            cross apply sys.dm_exec_sql_text(qs.sql_handle) st
cross apply sys.dm_exec_text_query_plan(qs.plan_handle, qs.statement_start_offset, qs.statement_end_offset) 
            where qs.sql_handle = @Sql_Handle and 
            qs.plan_generation_num <= @PlanGenNum + 1 ;
            fetch next from event_cursor into @Sql_Handle, @PlanGenNum;
      end
      close event_cursor;
      deallocate event_cursor;
end
go
 
exec analyze_perf_stats_trace_data 
go

-- Now let us look an example with recompiles:

create table t1 (a int, b int)
go
 
insert into t1 values (1, 1)
go
 
-- After 500 modifications to a table, any stored procedure referencing that table will 
-- need to be recompiled because the table statistics needs to be refreshed. 
create procedure RowCountDemo 
as 
begin       
    declare @i int;
    set @i = 0;
    while (@i < 1000) 
    begin
            insert into t1 values (@i, 2*@i - 50);
select a from t1 where a < 10 or ((b > 20 or a >=100) and (a < 10000)) group by a ;
            set @i = @i + 1; 
    end 
end 
go
 
exec RowCountDemo
go

