
-- find the size of SQLMGR cache
-- SQLMGR stores the actual text of the queries
select sum(bytes_used) as total_bytes_used 
from sys.dm_os_memory_objects where type = 'MEMOBJ_SQLMGR'
go

set quoted_identifier off
go
 
---this is an example for sql_handles, plan_handles interaction
if exists(select col1 from t1 where col2 > 5) declare @x int;
go
 
set quoted_identifier on
go
 
---this is an example for sql_handles, plan_handles interaction
if exists(select col1 from t1 where col2 > 5) declare @x int;
go
  
-- Use the following query to look to get the sql_handle, plan_handle and the sql text of the batch:
 
select st.text, qs. sql_handle, qs.plan_handle 
from sys.dm_exec_query_stats qs cross apply sys.dm_exec_sql_text(sql_handle) st
go

-- Usage of sys.dm_exec_cached_plans
-- The query below gives us the plan_handle, usecounts, size_in_bytes 
-- of the compiled plan, cacheobjtype, the compiled plan memory object address and the entire batch text:
 
select st.text, cp.plan_handle, cp.usecounts, cp.size_in_bytes, cp.cacheobjtype, cp.objtype, cp.memory_object_address
from sys.dm_exec_cached_plans cp 
cross apply sys.dm_exec_sql_text(cp.plan_handle) st
order by cp.usecounts desc
go

-- Usage of sys.dm_exec_cached_plan_dependent_objects
-- The query below gets the dependant objects for all 
-- compiled plans, their usecounts and their memory_object_address:
select st.text, cp.plan_handle, do.usecounts, do.cacheobjtype, 
do.memory_object_address
from sys.dm_exec_cached_plans cp
cross apply 
sys.dm_exec_sql_text(cp.plan_handle) st
cross apply 
sys.dm_exec_cached_plan_dependent_objects(cp.plan_handle) do
go

-- Consider the example below where the stored procedure is executed 
-- simultaneously from 10 connections: 
create procedure p1
as 
begin
      declare @x int;
      set @x = 0;
      while (@x < 10000000)
            set @x = @x + 1;
select t1.col2, t2.col2 from t1 join t2 on t1.col1 = t2.col1 here t1.col1 = 50000;
end
go

select plan_handle, memory_object_address, parent_address, type, name 
from dbo.CompPlanDetails
(0x0793A0C0, 
 0x050001003D299E5FB8A19307000000000000000000000000)
go

-- 10 simultaneous connections should == 10 MXCs (execution plans)
select st.text, do.usecounts, do.cacheobjtype, 
do.memory_object_address
from sys.dm_exec_cached_plans cp
cross apply 
sys.dm_exec_sql_text(cp.plan_handle) st
cross apply 
sys.dm_exec_cached_plan_dependent_objects(cp.plan_handle) do
where cp.plan_handle = 0x050001003D299E5FB8A19307000000000000000000000000
go

-- Usage of sys.dm_exec_query_requests
-- execution and resource consumption
-- currently executing statement in a 
-- stored procedure or batch
select top 10 substring(st.text, (er.statement_start_offset/2) + 1,
((case statement_end_offset 
when -1 
      then datalength(st.text) 
else 
      er.statement_end_offset 
end 
- er.statement_start_offset)/2) + 1) as statement_text
, *
from sys.dm_exec_requests er
cross apply sys.dm_exec_sql_text(er.sql_handle) st
order by total_elapsed_time desc
go

-- Usage of sys.dm_exec_query_stats
-- aggregate performance stats for cached query plans
-- top 10 queries with the highest CPU time
select top 10 substring(st.text, (qs.statement_start_offset/2) + 1,
((case statement_end_offset 
when -1 
      then datalength(st.text) 
else 
      qs.statement_end_offset 
end 
- qs.statement_start_offset)/2) + 1) as statement_text,
*
from sys.dm_exec_query_stats qs
cross apply sys.dm_exec_sql_text(qs.sql_handle) st
order by total_elapsed_time/execution_count desc
go


