

select col2 from t1 where col1 = 5
go
 
-- Now look at the query plans in sys.dm_exec_query_plan using the query below:
-- two rows, one autoparameterized, one is original query 
select st.text, qp.query_plan, cp.cacheobjtype, cp.objtype, cp.plan_handle 
from sys.dm_exec_cached_plans cp
cross apply sys.dm_exec_sql_text(cp.plan_handle) st
cross apply sys.dm_exec_query_plan(cp.plan_handle) qp
go

-- one row, autoparameterized only
select st.text, qp.query_plan, qs.plan_handle 
from sys.dm_exec_query_stats qs
cross apply sys.dm_exec_sql_text(qs.sql_handle) st
cross apply sys.dm_exec_query_plan(qs.plan_handle) qp
go

-- this query is unsafe autoparameterized
select value_in_use from sys.configurations where configuration_id = 16384
go
 
-- get query plan
select st.text, qp.query_plan, qs.plan_handle 
from sys.dm_exec_query_stats qs
cross apply sys.dm_exec_sql_text(qs.sql_handle) st
cross apply sys.dm_exec_query_plan(qs.plan_handle) qp
go

-- multistatement batch, one statement is autoparameterized
select * from t1
select col2 from t1 where col1 = 5
go
 
-- Now query the DMVs using the query below:
select st.text, qp.query_plan, cp.cacheobjtype, cp.objtype, cp.plan_handle 
from sys.dm_exec_cached_plans cp
cross apply sys.dm_exec_sql_text(cp.plan_handle) st
cross apply sys.dm_exec_query_plan(cp.plan_handle) qp
go

-- same batch gets different plan after showplan_xml turned on
set showplan_xml on
go
 
select * from t1
select col2 from t1 where col1 = 5
go

-- catalog procedure and look at query plan
create procedure p1 as
begin 
select * from t1
select col2 from t1 where col1 = 5
end
go
 
exec p1
go

-- Using sys.dm_exec_text_query_plan 
-- available in SP2 only
select substring(st.text, (qs.statement_start_offset/2) + 1,
((case statement_end_offset 
when -1 
      then datalength(st.text) 
else 
      qs.statement_end_offset 
end 
- qs.statement_start_offset)/2) + 1) as statement_text
, plan_handle, query_plan
from sys.dm_exec_query_stats qs
cross apply sys.dm_exec_sql_text(qs.sql_handle) st
cross apply sys.dm_exec_text_query_plan(qs.plan_handle, qs.statement_start_offset, qs.statement_end_offset) 
go

-- top five plans 
select top 5 total_worker_time/execution_count as avg_cpu_time,
substring(st.text, (qs.statement_start_offset/2) + 1,
((case statement_end_offset 
when -1 
      then datalength(st.text) 
else 
      qs.statement_end_offset 
end 
- qs.statement_start_offset)/2) + 1) as statement_text
, plan_handle, query_plan
from sys.dm_exec_query_stats qs
cross apply sys.dm_exec_sql_text(qs.sql_handle) st
cross apply sys.dm_exec_text_query_plan(qs.plan_handle, qs.statement_start_offset, qs.statement_end_offset) 
order by total_worker_time/execution_count desc;
go  

