
-- Number of CPUs
select cpu_count from sys.dm_os_sys_info 
go

-- Number of memory nodes
select distinct top 1 instance_name + 1 as number_of_nodes 
from sys.dm_os_performance_counters 
where object_name = 'SQLServer:Buffer Node'
go

-- if NUMA processors per node
select parent_node_id, scheduler_id from sys.dm_os_schedulers 
where scheduler_id < 255 
order by scheduler_id desc
go

-- total physical memory
select physical_memory_in_bytes/(1024.0*1024.0*1024.0) as physical_mem_in_gb 
from sys.dm_os_sys_info 
go

-- if NUMA, memory per node
select cntr_value 
from sys.dm_os_performance_counters 
where object_name = 'SQLServer:Buffer Node' and counter_name = 'Total pages'                                                      
go

-- /3GB or /UserVA
select 
     case 
           when virtual_memory_in_bytes / 1024 / (2048*1024) 
< 1 then 'No switch' 
           else '/3GB' 
     end 
from sys.dm_os_sys_info

-- max worker threads
sp_configure 'show advanced options', 1
go
reconfigure with override
go
 
sp_configure 'max worker threads'
go
 
--Each worker has a stack associated with it. The stack size can be found:
select stack_size_in_bytes/(1024.0 * 1024.0) as stack_size_in_MB from sys.dm_os_sys_info 
go

-- forced parameterization
select name, is_parameterization_forced from sys.databases
go

-- query mix
select usecounts, cacheobjtype, objtype, bucketid 
from sys.dm_exec_cached_plans cp 
where cacheobjtype = 'Compiled Plan' 
order by objtype
go

- collect the waitstats
select * from sys.dm_os_wait_stats 
order by waiting_tasks_count desc
go

-- SOS_RESERVEDMEMBLOCKLIST waits
-- RESOURCE_SEMAPHORE_QUERY_COMPILE waits
select * from sys.dm_os_wait_stats
order by wait_type
go
 
-- Sys.dm_os_memory_cache_counters has the multipages allocated in KB (multi_pages_kb):
select name, type, single_pages_kb, multi_pages_kb, 
single_pages_in_use_kb, multi_pages_in_use_kb 
from sys.dm_os_memory_cache_counters 
where type = 'CACHESTORE_SQLCP' or type = 'CACHESTORE_OBJCP'
go

-- Are queries getting compiled?
select usecounts, cacheobjtype, objtype, bucketid, text 
from sys.dm_exec_cached_plans cp cross apply 
sys.dm_exec_sql_text(cp.plan_handle) 
where cacheobjtype = 'Compiled Plan' 
order by objtype 
go

--Periodically look at the spins and collisions data 
create table t_spinlock_stats 
(spinlockname varchar(64), collisions int, 
spins int, [spins/collisions] int)
 
insert into t_spinlock_stats
exec ('dbcc sqlperf(spinlockstats)');
 
select * from t_spinlock_stats order by collisions desc
go

-- If the numbers of collisions (and spins) for SQL_MGR is relatively high, 
-- next examine the parameterized queries:
select usecounts, cacheobjtype, objtype, bucketid, text 
from sys.dm_exec_cached_plans cp cross apply sys.dm_exec_sql_text(cp.plan_handle) 
where cacheobjtype = 'Compiled Plan' and objtype = 'Prepared' 
order by bucketid
go

/*
If the numbers of collisions (and spins) for SPL_MUTEX is high, then examine your application code to see if queries are being executed under a security context other than the sysadmin. 
 
Examine the size (single_pages_kb) of the TokenAndPermUserStore cache either through dbcc memorystatus or through the sys.dm_os_memory_clerks DMV. The number of single_pages_kb should be very high.
*/ 

select type, name, single_pages_kb, 
multi_pages_kb, memory_clerk_address 
from sys.dm_os_memory_clerks where type = 
'USERSTORE_TOKENPERM'
go
 
-- Next examine the buckets_count in sys.dm_os_memory_cache_hash_tables and look for long hash bucket chains:
 
select type, name, cache_address, buckets_count, buckets_in_use_count, 
from sys.dm_os_memory_cache_hash_tables 
where type = 'USERSTORE_TOKENPERM'
go

-- If yes, next try to free the token and permissions cache (TokenAndPermUserStore) 
-- and see if that temporarily alleviates the problem:
 
dbcc freesystemcache ('TokenAndPermUserStore')
go
